package com.cesgroup.tomcat.plugin.bootstrap.listener;

import java.io.File;

import com.cesgroup.tomcat.plugin.util.Operate;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.monitor.FileAlterationListenerAdaptor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 文件监听类
 * @version 1.0.0 2018/07/24
 */
public class FileListener extends FileAlterationListenerAdaptor{
	
	protected  Logger logger = LoggerFactory.getLogger(getClass());
	// 原目录路径
	private String orig_root_path;
	// 目标目录路径
	private String dest_root_path;

	public FileListener(String orig_root_path,String dest_root_path) {
		this.orig_root_path = orig_root_path;
		this.dest_root_path = dest_root_path;
	}

	/**
	 * 目录发生变化时执行
	 * @param directory
	 */
	@Override
	public void onDirectoryChange(File directory) {
		logger.debug("onDirectoryChange"+directory);
	}
	/**
	 * 创建目录时执行
	 * @param directory
	 */
	@Override
	public void onDirectoryCreate(File directory) {
		logger.debug("onDirectoryCreate"+directory);
	}
	/**
	 * 删除目录时执行
	 * @param directory
	 */
	@Override
	public void onDirectoryDelete(File directory) {
		logger.debug("onFileCreate"+directory);
	}
	/**
	 * 目录下文件变化时执行
	 * @param file
	 */
	@Override
	public void onFileChange(File file) {
		logger.debug("onFileChange"+file);
		
		handleFile( file,Operate.CHANGE);
	}
	/**
	 * 目录下创建文件时执行
	 * @param file
	 */
	@Override
	public void onFileCreate(File file) {
		logger.debug("onFileCreate"+file);
		handleFile( file,Operate.ADD);
	}
	/**
	 * 目录下删除文件时执行
	 * @param file
	 */
	@Override
	public void onFileDelete(File file) {
		logger.debug("onFileDelete"+file);
		handleFile( file,Operate.DEL);
	}

	/**
	 * 文件变化时处理逻辑
	 * @param file
	 * @param operateType
	 */
	public void handleFile(File file,Operate operateType) {
		try {
			String suffix = file.getAbsolutePath().replace(orig_root_path, "");
			File newFile = new File(dest_root_path,suffix);
			newFile.getParentFile().mkdirs();
			switch(operateType) {
				case ADD:
					FileUtils.copyFile(file, newFile);
				case CHANGE:
					FileUtils.copyFile(file, newFile);
					break;
				case DEL:
					newFile.deleteOnExit();break;
			}	
			
		}catch(Exception e) {
			logger.error(e.getMessage(), e);
		}
		
	}
}
