package com.cesgroup.tomcat.plugin.bootstrap.module;

import com.cesgroup.tomcat.plugin.bootstrap.Container;
import org.apache.commons.compress.archivers.ArchiveInputStream;
import org.apache.commons.compress.archivers.ArchiveStreamFactory;
import org.apache.commons.compress.archivers.jar.JarArchiveEntry;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 *  项目启动时关联的模块
 * @version 1.0.0 2018/07/23
 */
public abstract class Module {
	
	protected  Logger logger = LoggerFactory.getLogger(getClass());
	// 项目路径
	protected String path;
	// web 容器
	protected Container container;
	// 不包含的文件
	protected List<String> excludes = new ArrayList<String>();
	// 包含的文件
	protected List<String> includes = new ArrayList<String>();

	public Module(String path, Container container) {
		this.path = path;
		this.container = container;
	}

	protected abstract void perfomCopyDirectory() ;

	/**
	 * 复制目录下文件
	 * @param srcDir 源目录
	 * @param destDir 目标目录
	 */
	public void copyDirectory(String srcDir,String destDir) {

		try {
			FileUtils.copyDirectory(new File(srcDir),new File(destDir),new FileFilter() {
				public boolean accept(File pathname) {
					
					if(!includes.isEmpty()) {
						for(String include:includes) {
							if(pathname.getAbsolutePath().contains(include)) {
								return true;
							}
						}
						return false;
					} else {
						for(String exclude:excludes) {
							if(pathname.getAbsolutePath().contains(exclude)) {
								return false;
							}
						}
						return true;
					}
					
				}
				
			} );
		} catch (Exception e) {
			logger.error(e.getMessage(),e);
		}
	}

	/**
	 * 解压文件
	 * @param warPath war包的路径
	 * @param unzipPath 解压位置
	 */
	public void unzip(String warPath,String unzipPath) {
		File warFile = new File(warPath);
		BufferedInputStream bufferedInputStream = null;
		try {
			bufferedInputStream = new BufferedInputStream(new FileInputStream(warFile));
			ArchiveInputStream in = new ArchiveStreamFactory().createArchiveInputStream(ArchiveStreamFactory.JAR,bufferedInputStream);
			JarArchiveEntry entry = null;
			while ((entry = (JarArchiveEntry) in.getNextEntry()) != null) {
				if (entry.isDirectory()) {
					new File(unzipPath, entry.getName()).mkdir();
				} else {
					OutputStream out = FileUtils.openOutputStream(new File(unzipPath, entry.getName()));
					IOUtils.copy(in, out);
					out.close();
				}
			}
			in.close();
		} catch(Exception e) {
			logger.error(e.getMessage(),e);
		}
	}

	public List<String> getExcludes() {
		return excludes;
	}

	public void setExcludes(List<String> excludes) {
		this.excludes = excludes;
	}

	public List<String> getIncludes() {
		return includes;
	}

	public void setIncludes(List<String> includes) {
		this.includes = includes;
	}
	
	 

}
