package com.cesgroup.tomcat.plugin.bootstrap.module;


import com.cesgroup.tomcat.plugin.bootstrap.Container;
import org.apache.commons.io.FileUtils;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

import javax.xml.parsers.DocumentBuilderFactory;
import java.io.File;
import java.io.IOException;

/**
 *  war包相关的模块
 * @version 1.0.0 2018/07/23
 */
public class WarModule extends Module {
	// 组Id
	private String group;
	// 构件Id
	private String artifactId;
	// 版本
	private String version;

	/**
	 * 按war路径的方式导入模块
	 * @param path war 相关的路径
	 * @param container web 容器
	 */
	public WarModule(String path,Container container) {
		super(path,container);
		perfomCopyDirectory();
	}

	/**
	 * 按maven 包的方式 导入模板
	 * @param group
	 * @param artifactId
	 * @param version
	 * @param container
	 */
	public WarModule(String group,String artifactId,String version,Container container) {
		
		this(null,container);
		
		this.group = group;
		
		this.artifactId = artifactId;
		
		this.version = version;
		
		try {
			this.path =   getWarPath();
			perfomCopyDirectory();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
	}

	/**
	 * 复制项目到启动容器中
	 */
	@Override
	protected void perfomCopyDirectory() {
		
		File unzipPath =  new File(new File(this.path).getParentFile(),"temp");
		// 解压相关的war文件
		unzip(path, unzipPath.getAbsolutePath());
		// 复制相关项目到启动容器中
		copyDirectory(container.getWebapp(), unzipPath.getAbsolutePath());
		
		try {
			FileUtils.deleteDirectory(unzipPath);
		} catch (IOException e) {

			logger.error(e.getMessage(), e);
		}
	}

	/**
	 * 取得war包所在的路径
	 * @return 返回相关路径
	 */
	private String getWarPath() {
		// 取得maven home 的路径
		String maven_home = System.getenv("MAVEN_HOME");
		// 取得默认的路径
		String repositoryPath =  new File(System.getProperty("user.home"),"/.m2/repository").getAbsolutePath();
		// 取得setting 的路径
		File setting = new File(maven_home,"conf/settings.xml");
		Document doc = null;
		try {
			 doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(setting);
		} catch (Exception e) {
			 logger.warn(e.getMessage(),e);
		}
		NodeList nodes = doc.getElementsByTagName("localRepository");
		
		if(nodes.getLength() == 1) {
			repositoryPath = nodes.item(0).getTextContent();
		}
		String groupPath =  new File(repositoryPath,group.replace(".", "/")).getAbsolutePath();
		
		String artifactIdPath = new File(groupPath,artifactId).getAbsolutePath();
		
		String versionPath = new File(artifactIdPath,version).getAbsolutePath();
		// 返回war 的路径
		return new File(versionPath,artifactId+"-"+version + ".war").getAbsolutePath();

	}


}
